﻿#include<hgl/network/SocketOutputStream.h>
#include<hgl/io/DataOutputStream.h>
#include<hgl/network/TCPSocket.h>
#include<hgl/log/LogInfo.h>
namespace hgl
{
    namespace network
    {
//         bool Write(io::DataOutputStream *dos,const sockaddr_in &addr)
//         {
//         #if (HGL_COMPILER == HGL_COMPILER_GNU)||(HGL_COMPILER == HGL_COMPILER_LLVM)
//             if(dos->WriteUint8((uint8 *)&(addr.sin_addr.s_addr),4)!=4)return(false);
//         #else
//             #if (HGL_OS == HGL_OS_Windows)
//                 if(dos->WriteUint8((uint8 *)&(addr.sin_addr.S_addr),4)!=4)return(false);
//             #endif//
//         #endif//
//
//             return dos->WriteUint16(addr.sin_port);
//         }
    }//namespace network

    namespace network
    {
        /**
        * 向socket中写入指定的字节数
        * @param buf 数据缓冲区
        * @param size 预想写入的字节数
        * @return 成功写入的字节数
        * @return -1 失败
        */
        int64 SocketOutputStream::Write(const void *buf,int64 size)
        {
            if(sock==-1)
            {
                LOG_ERROR(OS_TEXT("SocketOutputStream::Write() fatal error,sock=-1"));
                return(-1);
            }

            if(size==0)return(0);
            if(size<0)
            {
                LOG_ERROR(OS_TEXT("SocketOutputStream::Write() fatal error,size<0,sock=")+OSString::numberOf(sock));
                return(-3);
            }

            if(!buf)
            {
                LOG_ERROR(OS_TEXT("SocketOutputStream::Write() fatal error,buf=nullptr,sock=")+OSString::numberOf(sock));
                return(-2);
            }

            const int64 result=send(sock,(char *)buf,size,0);

            if(result>0)
            {
                total+=result;

//                LOG_INFO(OS_TEXT("Socket ")+OSString::numberOf(sock)+OS_TEXT(" send ")+OSString::numberOf(size)+OS_TEXT(" bytes ok,result ")+OSString(result)+OS_TEXT(" total send ")+OSString(total)+OS_TEXT(" bytes."));
            }
            else
            {
                int err=GetLastSocketError();

                LOG_INFO(OS_TEXT("Socket ")+OSString::numberOf(sock)+OS_TEXT(" send ")+OSString::numberOf(size)+OS_TEXT(" bytes failed,,error: ")+OSString::numberOf(err)+OS_TEXT(",")+GetSocketString(err));
            }

            return(result);
        }

        /**
        * 向socket中充分写入指定的字节数，无视超时，直接写完指定字节数为止
        * @param buf 数据缓冲区
        * @param size 预想写入的字节数
        * @return 成功写入的字节数
        * @return -1 失败
        */
        int64 SocketOutputStream::WriteFully(const void *buf,int64 size)
        {
            if(sock==-1)
            {
                LOG_ERROR(OS_TEXT("SocketOutputStream::WriteFully() fatal error,sock=-1"));
                return(-1);
            }

            if(size==0)return(0);

            if(size<0)
            {
                LOG_ERROR(OS_TEXT("SocketOutputStream::WriteFully() fatal error,size<0,sock=")+OSString::numberOf(sock));
                return(-3);
            }

            if(!buf)
            {
                LOG_ERROR(OS_TEXT("SocketOutputStream::WriteFully() fatal error,buf=nullptr,sock=")+OSString::numberOf(sock));
                return(-2);
            }

            int err;
            const os_char *err_str;
            char *p=(char *)buf;

//             const double start_time=GetDoubleTime();

#if HGL_OS == HGL_OS_Windows
            int result = 0;
            int left_bytes = size;
#else
            ssize_t result = 0;
            size_t left_bytes = size;
#endif//HGL_OS == HGL_OS_Windows

            while(left_bytes>0)
            {
                result=send(sock,p,left_bytes,0);

//                 LOG_ERROR(OS_TEXT("socket send  result, left_bytes:  ")+OSString::numberOf(sock)+OSString(" , ")+OSString((int)result)+OSString(" , ")+OSString((int)left_bytes));
                if(result==0)
                {
                    sock=-1;
                    break;
                }

                if(result<0)
                {
                    err=GetLastSocketError();

                    if(err==0
                     ||err==nseTimeOut
                     ||err==nseWouldBlock
                     ||err==nseInt
                    )
                        continue;

                    if(err==nseBrokenPipe)
                    {
                        sock=-1;
                        break;
                    }

                    err_str=GetSocketString(err);
                    if(err_str)
                    {
                        LOG_ERROR(OS_TEXT("SocketOutputStream::WriteFully error,Socket:")+OSString::numberOf(sock)+OS_TEXT(",error code=")+OSString::numberOf(err)+OS_TEXT(":")+OSString(err_str));
                    }
                    else
                    {
                        LOG_ERROR(OS_TEXT("SocketOutputStream::WriteFully error,Socket:")+OSString::numberOf(sock)+OS_TEXT(",error code=")+OSString::numberOf(err));
                    }
                    sock=-1;
                    break;
                }

                p+=result;
                left_bytes-=result;
            }

//             const double end_time=GetDoubleTime();

//             LOG_INFO(OS_TEXT("SocketOutputStream::WriteFully() time: ")+OSString(end_time-start_time));

            total+=(p-(char *)buf);

            return(p-(char *)buf);
        }

        int64 SocketOutputStream::Available()const
        {
            int send_buf_size=0;
            socklen_t len=sizeof(int);

            getsockopt(sock,SOL_SOCKET,SO_SNDBUF,(char *)&send_buf_size,&len);      //这个函数会出错，但我们不理会

            return send_buf_size;
        }
    }//namespace network
}//namespace hgl
